@@ -1,13 +1,14 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: July 16, 2017
+# Last Change: February 6, 2020
 # URL: https://humanfriendly.readthedocs.io
 
 """The main module of the `humanfriendly` package."""
 
 # Standard library modules.
 import collections
+import datetime
 import decimal
 import multiprocessing
 import numbers
@@ -39,7 +40,7 @@ from humanfriendly.prompts import prompt_for_choice  # NOQA
 from humanfriendly.compat import is_string, monotonic
 
 # Semi-standard module versioning.
-__version__ = '4.4'
+__version__ = '5.0'
 
 # Spinners are redrawn at most this many seconds.
 minimum_spinner_interval = 0.2
@@ -52,6 +53,7 @@ erase_line_code = '\r\x1b[K'
 hide_cursor_code = '\x1b[?25l'
 show_cursor_code = '\x1b[?25h'
 
+# Named tuples to define units of size.
 SizeUnit = collections.namedtuple('SizeUnit', 'divider, symbol, name')
 CombinedUnit = collections.namedtuple('CombinedUnit', 'decimal, binary')
 
@@ -62,6 +64,9 @@ disk_size_units = (
     CombinedUnit(SizeUnit(1000**3, 'GB', 'gigabyte'), SizeUnit(1024**3, 'GiB', 'gibibyte')),
     CombinedUnit(SizeUnit(1000**4, 'TB', 'terabyte'), SizeUnit(1024**4, 'TiB', 'tebibyte')),
     CombinedUnit(SizeUnit(1000**5, 'PB', 'petabyte'), SizeUnit(1024**5, 'PiB', 'pebibyte')),
+    CombinedUnit(SizeUnit(1000**6, 'EB', 'exabyte'), SizeUnit(1024**6, 'EiB', 'exbibyte')),
+    CombinedUnit(SizeUnit(1000**7, 'ZB', 'zettabyte'), SizeUnit(1024**7, 'ZiB', 'zebibyte')),
+    CombinedUnit(SizeUnit(1000**8, 'YB', 'yottabyte'), SizeUnit(1024**8, 'YiB', 'yobibyte')),
 )
 
 # Common length size units, used for formatting and parsing.
@@ -104,11 +109,56 @@ def coerce_boolean(value):
             return False
         else:
             msg = "Failed to coerce string to boolean! (%r)"
-            raise ValueError(msg % value)
+            raise ValueError(format(msg, value))
     else:
         return bool(value)
 
 
+def coerce_pattern(value, flags=0):
+    """
+    Coerce strings to compiled regular expressions.
+
+    :param value: A string containing a regular expression pattern
+                  or a compiled regular expression.
+    :param flags: The flags used to compile the pattern (an integer).
+    :returns: A compiled regular expression.
+    :raises: :exc:`~exceptions.ValueError` when `value` isn't a string
+             and also isn't a compiled regular expression.
+    """
+    if is_string(value):
+        value = re.compile(value, flags)
+    else:
+        empty_pattern = re.compile('')
+        pattern_type = type(empty_pattern)
+        if not isinstance(value, pattern_type):
+            msg = "Failed to coerce value to compiled regular expression! (%r)"
+            raise ValueError(format(msg, value))
+    return value
+
+
+def coerce_seconds(value):
+    """
+    Coerce a value to the number of seconds.
+
+    :param value: An :class:`int`, :class:`float` or
+                  :class:`datetime.timedelta` object.
+    :returns: An :class:`int` or :class:`float` value.
+
+    When `value` is a :class:`datetime.timedelta` object the
+    :func:`~datetime.timedelta.total_seconds()` method is called.
+    On Python 2.6 this method is not available so it is emulated.
+    """
+    if isinstance(value, datetime.timedelta):
+        if hasattr(value, 'total_seconds'):
+            return value.total_seconds()
+        else:
+            return (value.microseconds + (value.seconds + value.days * 24 * 3600) * 10**6) / 10**6
+    if not isinstance(value, numbers.Number):
+        msg = "Failed to coerce value to number of seconds! (%r)"
+        raise ValueError(format(msg, value))
+    return value
+
+
 def format_size(num_bytes, keep_width=False, binary=False):
     """
     Format a byte count as a human readable file size.
@@ -205,7 +255,7 @@ def parse_size(size, binary=False):
                     return int(tokens[0] * (unit.binary.divider if binary else unit.decimal.divider))
     # We failed to parse the size specification.
     msg = "Failed to parse size! (input %r was tokenized as %r)"
-    raise InvalidSize(msg % (size, tokens))
+    raise InvalidSize(format(msg, size, tokens))
 
 
 def format_length(num_metres, keep_width=False):
@@ -274,7 +324,7 @@ def parse_length(length):
                     return tokens[0] * unit['divider']
     # We failed to parse the length specification.
     msg = "Failed to parse length! (input %r was tokenized as %r)"
-    raise InvalidLength(msg % (length, tokens))
+    raise InvalidLength(format(msg, length, tokens))
 
 
 def format_number(number, num_decimals=2):
@@ -341,7 +391,7 @@ def round_number(count, keep_width=False):
     text = '%.2f' % float(count)
     if not keep_width:
         text = re.sub('0+$', '', text)
-        text = re.sub('\.$', '', text)
+        text = re.sub(r'\.$', '', text)
     return text
 
 
@@ -349,13 +399,14 @@ def format_timespan(num_seconds, detailed=False, max_units=3):
     """
     Format a timespan in seconds as a human readable string.
 
-    :param num_seconds: Number of seconds (integer or float).
+    :param num_seconds: Any value accepted by :func:`coerce_seconds()`.
     :param detailed: If :data:`True` milliseconds are represented separately
                      instead of being represented as fractional seconds
                      (defaults to :data:`False`).
     :param max_units: The maximum number of units to show in the formatted time
                       span (an integer, defaults to three).
     :returns: The formatted timespan as a string.
+    :raise: See :func:`coerce_seconds()`.
 
     Some examples:
 
@@ -373,6 +424,7 @@ def format_timespan(num_seconds, detailed=False, max_units=3):
     >>> format_timespan(week * 52 + day * 2 + hour * 3)
     '1 year, 2 days and 3 hours'
     """
+    num_seconds = coerce_seconds(num_seconds)
     if num_seconds < 60 and not detailed:
         # Fast path.
         return pluralize(round_number(num_seconds), 'second')
@@ -461,7 +513,7 @@ def parse_timespan(timespan):
                     return float(tokens[0]) * unit['divider']
     # We failed to parse the timespan specification.
     msg = "Failed to parse timespan! (input %r was tokenized as %r)"
-    raise InvalidTimespan(msg % (timespan, tokens))
+    raise InvalidTimespan(format(msg, timespan, tokens))
 
 
 def parse_date(datestring):
@@ -532,7 +584,7 @@ def parse_date(datestring):
             return (year, month, day, 0, 0, 0)
     except Exception:
         msg = "Invalid date! (expected 'YYYY-MM-DD' or 'YYYY-MM-DD HH:MM:SS' but got: %r)"
-        raise InvalidDate(msg % datestring)
+        raise InvalidDate(format(msg, datestring))
 
 
 def format_path(pathname):
@@ -635,6 +687,32 @@ class Timer(object):
             self.total_time += monotonic() - self.start_time
             self.start_time = 0.0
 
+    def sleep(self, seconds):
+        """
+        Easy to use rate limiting of repeating actions.
+
+        :param seconds: The number of seconds to sleep (an
+                        integer or floating point number).
+
+        This method sleeps for the given number of seconds minus the
+        :attr:`elapsed_time`. If the resulting duration is negative
+        :func:`time.sleep()` will still be called, but the argument
+        given to it will be the number 0 (negative numbers cause
+        :func:`time.sleep()` to raise an exception).
+
+        The use case for this is to initialize a :class:`Timer` inside
+        the body of a :keyword:`for` or :keyword:`while` loop and call
+        :func:`Timer.sleep()` at the end of the loop body to rate limit
+        whatever it is that is being done inside the loop body.
+
+        For posterity: Although the implementation of :func:`sleep()` only
+        requires a single line of code I've added it to :mod:`humanfriendly`
+        anyway because now that I've thought about how to tackle this once I
+        never want to have to think about it again :-P (unless I find ways to
+        improve this).
+        """
+        time.sleep(max(0, seconds - self.elapsed_time))
+
     @property
     def elapsed_time(self):
         """
@@ -933,7 +1011,7 @@ class InvalidDate(Exception):
     >>> parse_date('2013-06-XY')
     Traceback (most recent call last):
       File "humanfriendly.py", line 206, in parse_date
-        raise InvalidDate, msg % datestring
+        raise InvalidDate(format(msg, datestring))
     humanfriendly.InvalidDate: Invalid date! (expected 'YYYY-MM-DD' or 'YYYY-MM-DD HH:MM:SS' but got: '2013-06-XY')
     """
 
@@ -949,7 +1027,7 @@ class InvalidSize(Exception):
     >>> parse_size('5 Z')
     Traceback (most recent call last):
       File "humanfriendly/__init__.py", line 267, in parse_size
-        raise InvalidSize(msg % (size, tokens))
+        raise InvalidSize(format(msg, size, tokens))
     humanfriendly.InvalidSize: Failed to parse size! (input '5 Z' was tokenized as [5, 'Z'])
     """
 
@@ -965,7 +1043,7 @@ class InvalidLength(Exception):
     >>> parse_length('5 Z')
     Traceback (most recent call last):
       File "humanfriendly/__init__.py", line 267, in parse_length
-        raise InvalidLength(msg % (length, tokens))
+        raise InvalidLength(format(msg, length, tokens))
     humanfriendly.InvalidLength: Failed to parse length! (input '5 Z' was tokenized as [5, 'Z'])
     """
 
@@ -981,6 +1059,6 @@ class InvalidTimespan(Exception):
     >>> parse_timespan('1 age')
     Traceback (most recent call last):
       File "humanfriendly/__init__.py", line 419, in parse_timespan
-        raise InvalidTimespan(msg % (timespan, tokens))
+        raise InvalidTimespan(format(msg, timespan, tokens))
     humanfriendly.InvalidTimespan: Failed to parse timespan! (input '1 age' was tokenized as [1, 'age'])
     """
